unit uOpenCalc;

interface

uses Graphics, Variants, ComObj, Windows;

type
  TOpenCP=record
    StartCol:integer;
    StartRow:integer;
    EndCol:integer;
    EndRow:integer;
  end;
  TOpenBS=set of (obsLeft,obsTop,obsRight,obsBottom);
  TOpenFS=set of (ofsBold,ofsUnderline,ofsItalic,ofsStrikeOut);
  TOpenHA=(ohaDefault,ohaLeft,ohaCenter,ohaRight);
  TOpenVA=(ovaDefault,ovaTop,ovaCenter,ovaBottom);

  TOpenCalc=class
    private
      FServiceManager:Variant;
      FDesktop:Variant;
      FDocument:Variant;
      FScreenUpdating:boolean;
      function ConvertToURL(FileName:string):string;
      function IsSheetExist(SheetName:string):boolean;
      procedure SetScreenUpdating(Value:boolean);
      procedure SetFontColor(CPos:TOpenCP; const Value:TColor);
      procedure SetFontSize(CPos: TOpenCP; const Value: integer);
      procedure SetFontName(CPos: TOpenCP; const Value: string);
      procedure SetFontStyle(CPos: TOpenCP; const Value: TOpenFS);
      procedure SetFontCharSet(CPos: TOpenCP; const Value: string);
      procedure SetBackgroundColor(CPos: TOpenCP; const Value: TColor);
      function GetSheetsCount:integer;
      procedure SetColWidth(Col:integer; const Value:integer);
      procedure SetRowHeight(Row: integer; const Value: integer);
      procedure SetCellDateCR(Col, Row: integer; const Value: TDateTime);
      procedure SetCellDateNM(CellNum: string; const Value: TDateTime);
      procedure SetCellFormulaCR(Col, Row: integer; const Value: string);
      procedure SetCellFormulaNM(CellNum: string; const Value: string);
      procedure SetCellNumberCR(Col, Row: integer; const Value: double);
      procedure SetCellNumberNM(CellNum: string; const Value: double);
      procedure SetCellTextCR(Col, Row: integer; const Value: string);
      procedure SetCellTextNM(CellNum: string; const Value: string);
      procedure SetHorAlignment(CPos: TOpenCP; const Value: TOpenHA);
      procedure SetVerAlignment(CPos: TOpenCP; const Value: TOpenVA);
      function GetSheetName(SheetNum: integer): string;
      procedure SetSheetName(SheetNum: integer; const Value: string);
      procedure SetCellsFormatCR(CPos: TOpenCP; const Value: string);
      procedure SetCellsFormatNM(CellRange: string; const Value: string);
    public
      constructor Create;
      destructor Destroy;override;
      function IsConnected:boolean;
      function Connect:boolean;
      procedure Disconnect;
      procedure OpenDocument(FileName:string);
      procedure InsertSheet(NewSheetName:string; SheetNum:integer);
      procedure SetActiveSheet(SheetNum:integer);
      procedure SetBorders(CPos:TOpenCP; Border:TOpenBS; Color:TColor; Width:integer);
      procedure MergeCells(CPos:TOpenCP);
      procedure SetCellByArray(Col,Row:integer; Value:variant);
      //property
      property SheetName[SheetNum:integer]:string read GetSheetName write SetSheetName;
      property CellDateByCR[Col,Row:integer]:TDateTime write SetCellDateCR;
      property CellDateByNM[CellNum:string]:TDateTime write SetCellDateNM;
      property CellFormulaByCR[Col,Row:integer]:string write SetCellFormulaCR;
      property CellFormulaByNM[CellNum:string]:string write SetCellFormulaNM;
      property CellNumberByCR[Col,Row:integer]:double write SetCellNumberCR;
      property CellNumberByNM[CellNum:string]:double write SetCellNumberNM;
      property CellTextByCR[Col,Row:integer]:string write SetCellTextCR;
      property CellTextByNM[CellNum:string]:string write SetCellTextNM;
      property CellsFormatByCR[CPos:TOpenCP]:string write SetCellsFormatCR;
      property CellsFormatByNM[CellRange:string]:string write SetCellsFormatNM;
      property ColWidth[Col:integer]:integer write SetColWidth;
      property RowHeight[Row:integer]:integer write SetRowHeight;
      property SheetsCount:integer read GetSheetsCount;
      property ScreenUpdating:boolean read FScreenUpdating write SetScreenUpdating;
      property FontColor[CPos:TOpenCP]:TColor write SetFontColor;
      property FontSize[CPos:TOpenCP]:integer write SetFontSize;
      property FontName[CPos:TOpenCP]:string write SetFontName;
      property FontStyle[CPos:TOpenCP]:TOpenFS write SetFontStyle;
      property FontCharSet[CPos:TOpenCP]:string write SetFontCharSet;
      property BackgroundColor[CPos:TOpenCP]:TColor write SetBackgroundColor;
      property HorAlignment[CPos:TOpenCP]:TOpenHA write SetHorAlignment;
      property VerAlignment[CPos:TOpenCP]:TOpenVA write SetVerAlignment;
  end;

function CPos(StartCol,StartRow,EndCol,EndRow:integer):TOpenCP;

implementation

function CPos(StartCol,StartRow,EndCol,EndRow:integer):TOpenCP;
begin
  Result.StartCol:=StartCol;
  Result.StartRow:=StartRow;
  Result.EndCol:=EndCol;
  Result.EndRow:=EndRow;
end;

{ TOpenCalc }

constructor TOpenCalc.Create;
begin
  inherited;
  FServiceManager:=Unassigned;
  FDesktop:=Unassigned;
  FDocument:=Unassigned;
end;

destructor TOpenCalc.Destroy;
begin
  Disconnect;
  inherited;
end;

function TOpenCalc.ConvertToURL(FileName: string): string;
var
  URL:string;
  i:integer;
  ch:char;
begin
  URL:='';
  for i:=1 to Length(FileName) do
    begin
      ch:=FileName[i];
      case ch of
        ' ':URL:=URL+'%20';
        '\':URL:=URL+'/';
      else
        URL:=URL+ch;
      end;
    end;
  Result:='file://localhost/'+URL;
end;

function TOpenCalc.Connect: boolean;
begin
  Result:=true;
  try
    FServiceManager:=CreateOleObject('com.sun.star.ServiceManager');
    if (VarType(FServiceManager)=varDispatch) then
      FDesktop:=FServiceManager.CreateInstance('com.sun.star.frame.Desktop');
  except
    FDesktop:=Unassigned;
    FServiceManager:=Unassigned;
    Result:=false;
  end;
end;

procedure TOpenCalc.Disconnect;
begin
  FDocument:=Unassigned;
  FDesktop:=Unassigned;
  FServiceManager:=Unassigned;
end;

procedure TOpenCalc.SetBorders(CPos:TOpenCP; Border:TOpenBS; Color:TColor; Width:integer);
var
  Tmp:variant;
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Tmp:=FServiceManager.Bridge_GetStruct('com.sun.star.table.BorderLine');
  Tmp.Color:=RGB(GetBValue(Color),GetGValue(Color),GetRValue(Color));
  Tmp.LineDistance:=0;
  Tmp.InnerLineWidth:=0;

  Tmp.OuterLineWidth:=Ord(obsLeft in Border)*Width;
  Cell.LeftBorder:=Tmp;

  Tmp.OuterLineWidth:=Ord(obsTop in Border)*Width;
  Cell.TopBorder:=Tmp;

  Tmp.OuterLineWidth:=Ord(obsRight in Border)*Width;
  Cell.RightBorder:=Tmp;

  Tmp.OuterLineWidth:=Ord(obsBottom in Border)*Width;
  Cell.BottomBorder:=Tmp;
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetActiveSheet(SheetNum: integer);
begin
  FDocument.GetCurrentController.SetActiveSheet(FDocument.GetSheets.GetByIndex(SheetNum));
end;

function TOpenCalc.GetSheetsCount: integer;
begin
  Result:=FDocument.GetSheets.GetCount;
end;

procedure TOpenCalc.InsertSheet(NewSheetName: string; SheetNum: integer);
begin
  if (not IsSheetExist(NewSheetName)) then
    FDocument.GetSheets.InsertNewByName(NewSheetName, SheetNum);
  SetActiveSheet(SheetNum);
end;

function TOpenCalc.IsConnected: boolean;
begin
  Result:=(not VarIsEmpty(FDesktop)) or (VarIsNull(FDesktop)) or (VarIsClear(FDesktop));
end;

function TOpenCalc.IsSheetExist(SheetName: string): boolean;
begin
  Result:=FDocument.GetSheets.HasByName(SheetName);
end;

procedure TOpenCalc.OpenDocument(FileName: string);
var
  FilePath:string;
  VariantArray:variant;
  Bounds:array [0..1] of integer;
begin
  if (FileName='') then
    FilePath:='private:factory/scalc'
  else
    FilePath:=ConvertToURL(FileName);
  Bounds[0]:=0;
  Bounds[1]:=-1;
  VariantArray:=VarArrayCreate(Bounds, varVariant);
  FDocument:=FDesktop.LoadComponentFromURL(FilePath, '_blank', 0, VariantArray);
  VariantArray:=Unassigned;
end;

procedure TOpenCalc.SetScreenUpdating(Value: boolean);
begin
  if Value then
    begin
      FDocument.UnLockControllers;
      FDocument.RemoveActionLock;
    end
  else
    begin
      FDocument.LockControllers;
      FDocument.AddActionLock;
    end;
  FScreenUpdating:=Value;
end;

procedure TOpenCalc.SetFontColor(CPos:TOpenCP; const Value:TColor);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.CharColor:=RGB(GetBValue(Value),GetGValue(Value),GetRValue(Value));
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetFontSize(CPos: TOpenCP; const Value: integer);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.CharHeight:=Value;
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetFontName(CPos: TOpenCP; const Value: string);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.CharFontName:=Value;
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetFontStyle(CPos: TOpenCP; const Value: TOpenFS);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  if ofsBold in Value then
    Cell.CharWeight:=150
  else
    Cell.CharWeight:=0;
  Cell.CharUnderline:=ofsUnderline in Value;
  Cell.CharPosture:=ofsItalic in Value;
  Cell.CharStrikeout:=ofsStrikeOut in Value;
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetFontCharSet(CPos: TOpenCP; const Value: string);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.CharFontCharSet:=Value;
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetBackgroundColor(CPos: TOpenCP; const Value: TColor);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.CellBackColor:=RGB(GetBValue(Value),GetGValue(Value),GetRValue(Value));
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetColWidth(Col:integer; const Value:integer);
var
  Column:variant;
begin
  Column:=FDocument.GetCurrentController.GetActiveSheet.GetColumns.GetByIndex(Col);
  Column.OptimalWidth:=Value<0;
  if Value>-1 then
    Column.Width:=Value;
  Column:=Unassigned;
end;

procedure TOpenCalc.SetRowHeight(Row:integer; const Value:integer);
var
  FRow:variant;
begin
  FRow:=FDocument.GetCurrentController.GetActiveSheet.GetRows.GetByIndex(Row);
  FRow.OptimalHeight:=Value<0;
  if Value>-1 then
    FRow.Height:=Value;
  FRow:=Unassigned;
end;

procedure TOpenCalc.SetCellDateCR(Col,Row:integer; const Value:TDateTime);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellByPosition(Col,Row).SetString(Value);
end;

procedure TOpenCalc.SetCellDateNM(CellNum:string; const Value:TDateTime);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByName(CellNum).SetString(Value);
end;

procedure TOpenCalc.SetCellFormulaCR(Col,Row:integer; const Value:string);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellByPosition(Col,Row).SetFormula(Value);
end;

procedure TOpenCalc.SetCellFormulaNM(CellNum:string; const Value:string);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByName(CellNum).SetFormula(Value);
end;

procedure TOpenCalc.SetCellNumberCR(Col,Row:integer; const Value:double);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellByPosition(Col,Row).SetValue(Value);
end;

procedure TOpenCalc.SetCellNumberNM(CellNum:string; const Value:double);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByName(CellNum).SetValue(Value);
end;

procedure TOpenCalc.SetCellTextCR(Col, Row: integer; const Value: string);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellByPosition(Col,Row).SetString(Value);
end;

procedure TOpenCalc.SetCellTextNM(CellNum: string; const Value: string);
begin
  FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByName(CellNum).SetString(Value);
end;

procedure TOpenCalc.MergeCells(CPos: TOpenCP);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.Merge(true);
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetHorAlignment(CPos: TOpenCP; const Value: TOpenHA);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.HoriJustify:=Value;
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetVerAlignment(CPos: TOpenCP; const Value: TOpenVA);
var
  Cell:variant;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  Cell.VertJustify:=Value;
  Cell:=Unassigned;
end;

function TOpenCalc.GetSheetName(SheetNum: integer): string;
begin
  Result:=FDocument.GetSheets.GetByIndex(SheetNum).Name;
end;

procedure TOpenCalc.SetSheetName(SheetNum: integer; const Value: string);
begin
  FDocument.GetSheets.GetByIndex(SheetNum).Name:=Value
end;

procedure TOpenCalc.SetCellByArray(Col,Row:integer; Value:variant);
var
  Cell:variant;
  x,y:integer;
begin
  x:=VarArrayHighBound(Value,1)-VarArrayLowBound(Value,1);
  y:=VarArrayHighBound(Value,2)-VarArrayLowBound(Value,2);
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(Col,Row,Col+x,Row+y);
  Cell.SetDataArray(Value);
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetCellsFormatCR(CPos:TOpenCP; const Value:string);
var
  NF:variant;
  Cell:variant;
  Loc:variant;
  FKey:integer;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByPosition(CPos.StartCol,CPos.StartRow,CPos.EndCol,CPos.EndRow);
  NF:=FDocument.GetNumberFormats;
  Loc:=FServiceManager.Bridge_GetStruct('com.sun.star.lang.Locale');
  FKey:=NF.QueryKey(Value,Loc,false);
  if FKey=-1 then
    FKey:=NF.AddNew(Value,Loc);
  Cell.NumberFormat:=FKey;
  Loc:=Unassigned;
  NF:=Unassigned;
  Cell:=Unassigned;
end;

procedure TOpenCalc.SetCellsFormatNM(CellRange:string; const Value:string);
var
  NF:variant;
  Cell:variant;
  Loc:variant;
  FKey:integer;
begin
  Cell:=FDocument.GetCurrentController.GetActiveSheet.GetCellRangeByName(CellRange);
  NF:=FDocument.GetNumberFormats;
  Loc:=FServiceManager.Bridge_GetStruct('com.sun.star.lang.Locale');
  FKey:=NF.QueryKey(Value,Loc,false);
  if FKey=-1 then
    FKey:=NF.AddNew(Value,Loc);
  Cell.NumberFormat:=FKey;
  Loc:=Unassigned;
  NF:=Unassigned;
  Cell:=Unassigned;
end;

end.

